import sys
import os
import numpy as np

OUT_DIR="csv"
SMALL=False

COUNTERS = [
    'alloc_dram_count',
    'alloc_dram_word' ,
    'alloc_nvm_count' ,
    'alloc_nvm_word' ,
    'persistent_count',
    'persistent_word',
    'ensure_recoverable',
    'handshake',
    'barrier_full',
    'barrier_async',
    'barrier_no'
]

xPROGRAM = [
    'avrora',
    'fop',
    'h2',
    'jython',
    'luindex',
    'lusearch',
    'lusearch-fix',
    'pmd',
    'sunflow',
    'xalan',
    'HAMT-1',
    'HAMT-10',
    'HAMTS-1',
    'HAMTS-10',
    'CHAMP-1',
    'CHAMP-10',
    'xml',
    'xml-custom-hashmap',
    'xml-tu',
]

PROGRAM = [
    'xml-tu',
    'HAMT-1',
    'HAMT-10',
    'CHAMP-1',
    'CHAMP-10',
    'avrora',
    'fop',
    'h2',
    'jython',
    'luindex',
    'lusearch',
    'pmd',
    'sunflow',
    'xalan',
]

PROGRAM_NAME = {
    "xml-tu": "XML"
}

def config_string(c):
    return "%(setting)s-%(vm)s-%(analysis)s" % c

def load_data(dir_name, configs):
    def load_data1(b, c):
        p = {
            "s": c["setting"],
            "v": c["vm"],
            "a": c["analysis"],
            "b": b
        }
        file_name = dir_name + "/count_%(s)s-%(v)s-%(a)s_%(b)s.txt" % p
        counts = {}
        if os.path.isfile(file_name):
            with open(file_name) as f:
                for line in f.readlines():
                    print(line)
                    print(file_name)
                    v1, v2, k = line.strip().split('\t')
                    if v1 == "-" or v2 == "-":
                        # print(file_name+" is wrong: one of v1 and v2 is invalid")
                        continue
                    counts[k] = str(int(v2) - int(v1))
        return counts

    data = {}
    for b in PROGRAM:
        data[b] = {}
        for c in configs:
            data[b][config_string(c)] = load_data1(b, c)
    return data

def load_time(dir_name, configs):
    def load_time1(b, c):
        p = {
            "s": c["setting"],
            "v": c["vm"],
            "a": c["analysis"],
            "b": b
        }
        file_name= dir_name + "/%(s)s_%(b)s_%(v)s_%(a)s.txt" % p
        numbers = []
        try:
            with open(file_name) as f:
                for line in f.readlines():
                    numbers.append(float(line))
            if SMALL:
                return np.mean(numbers[1:])
            else:
                return np.mean(numbers[5:])
        except:
            return 0
    
    data = {}
    for b in PROGRAM:
        data[b] = {}
        for c in configs:
            data[b][config_string(c)] = load_time1(b, c)

    return data

def output_data(data, configs):
    def do_output(b, data):
        file_name = OUT_DIR + "/count_" + b + ".csv"
        with open(file_name, "w") as f:
            f.write("counter,")
            f.write(",".join([config_string(c) for c in configs]))
            f.write("\n")
            for counter in COUNTERS:
                row = []
                for c in configs:
                    if counter in data[config_string(c)]:
                        row.append(data[config_string(c)][counter])
                    else:
                        row.append("")
                f.write(counter)
                f.write(",")
                f.write(",".join(row))
                f.write("\n")

    for b in PROGRAM:
        print(b)
        do_output(b, data[b])

def output_summary(data, time):
    COLUMNS = [
        ("handshake", "all-cw-none"),
        ("persistent_count", "all-cw-none"),
        ("persistent_word", "all-cw-none"),
        ("ensure_recoverable", "all-cw-none"),
    ]

    with open(OUT_DIR + "/summary.csv", "w") as f:
        f.write("benchmark,")
        f.write(",".join([x for x, _ in COLUMNS]))
        f.write(",PCperC")
        f.write(",PWperC")
        f.write(",retry")
        f.write(",cwTime")
        f.write("\n")
        for b in PROGRAM:
            if "handshake" not in data[b]["all-cw-none"]:
                continue
            print(b)
            print(data[b])
            d = {x: float(data[b][y][x]) for x, y in COLUMNS}
            row = [b] + [d[x] for x, y in COLUMNS]
            row.append(d["persistent_count"] / d["ensure_recoverable"])
            row.append(d["persistent_word"] / d["ensure_recoverable"])
            row.append(d["handshake"] - d["ensure_recoverable"])
            row.append(time[b]["all-cw-none"])
            f.write(",".join([str(x) for x in row]))
            f.write("\n")

def make_summary(data, time):
    header = []
    body = []
    
    header.append("benchmark")
    header.append("time")
#    header.append("M write")
#    header.append("M write/sec")
    header.append("K call")
    header.append("K call/sec")
    header.append("Mw persist")
    header.append("w persist/call")
    header.append("Kw persist/sec")

    c = "all-cw-none"
    
    for b in PROGRAM:
        xtime = time[b][c]/10**3
        xcall = float(data[b][c]["ensure_recoverable"])
        row = []
        row.append(PROGRAM_NAME[b] if b in PROGRAM_NAME else b)
        row.append(time[b][c]/10**3)
#        row.append(float(data[b][c]["barrier_full"])/10**6)
#        row.append(float(data[b][c]["barrier_full"])/xtime/10**6)
        row.append(float(data[b][c]["ensure_recoverable"])/10**3)
        row.append(float(data[b][c]["ensure_recoverable"])/xtime/10**3)
        row.append(float(data[b][c]["persistent_word"])/10**6)
        row.append(float(data[b][c]["persistent_word"])/xcall)
        row.append(float(data[b][c]["persistent_word"])/xtime/10**3)
        body.append(row)

    return (header, body)

def output_csv(file_name, header, body):
    with open(file_name, "w") as f:
        f.write(",".join(header))
        f.write("\n")

        for row in body:
            f.write(row[0] + ", ")
            f.write(",".join([str(x) for x in row[1:]]))
            f.write("\n")

def output_tex(file_name, header, body):
    with open(file_name, "w") as f:
        f.write("% ")
        f.write(",".join(header))
        f.write("\n")

        for row in body:
            f.write(row[0] + " & ")
            f.write(" & ".join(["%.1f" % x for x in row[1:]]))
            f.write("\\\\\n")

def main():
    #dir_name = sys.argv[1]
    dir_name = "results"
    os.makedirs(OUT_DIR, exist_ok=True)

    configs = [
        {"setting": "none", "vm": "original", "analysis": "none"},
        {"setting": "none", "vm": "ww", "analysis": "none"},
        {"setting": "none", "vm": "cw", "analysis": "none"},
        {"setting": "all",  "vm": "ww", "analysis": "none"},
        {"setting": "all",  "vm": "cw", "analysis": "none"},
        {"setting": "all",  "vm": "cw", "analysis": "h"},
        {"setting": "all",  "vm": "cw", "analysis": "b"},
        {"setting": "all",  "vm": "cw", "analysis": "hb"}
    ]

    configs2 = [
        {"setting": "all",  "vm": "cw", "analysis": "none"},
        {"setting": "none", "vm": "original", "analysis": "none"},
        {"setting": "all",  "vm": "ww", "analysis": "none"},
    ]

    data = load_data(dir_name, configs)
    time = load_time("results", configs2)

    output_data(data, configs)
#    output_summary(data, time)
    header, body = make_summary(data, time)
    output_csv(OUT_DIR + "/table.csv", header, body)
    output_tex(OUT_DIR + "/table.tex", header, body)

if len(sys.argv) >= 2 and sys.argv[1] == "-small":
    SMALL=True

main()
               
