# execute this file in "ccjava" dir

# cwx: cw without handshake (unsafe)
# wwx: ww without mfence (unsafe)

VMDIR=vms
REPO_WW=ccjava-src
REPO_CW=ccjava-src

if [ x$NVM_FILE = "x" ];then
    echo \$NVM_FILE is not set
    exit 1
fi
if [ \! -f $NVM_FILE ]; then
    echo \$NVM_FILE is not a regular file
    exit 1
fi


function checkout_branch() {
    name=$1

    mkdir -p $VMDIR

    case $name in
	none-original-none)
	    repo=$REPO_WW
	    commit="3b56f0b15cee132d6366e8ec03e05c918a42b4f1"
	    ;;
	*-ww-* | *-wwx-*)
	    repo=$REPO_WW
	    # zhang-jit-stable
	    commit="b039e12b48ef5ff441ab2b50e1fd9fea86fbdc35"
	    ;;
	*-cw-* | *-cw1-* | *-cwx-*)
	    repo=$REPO_CW
	    commit="2abad8f6676b01d69be5c7a44ba37a9882c71763"
	    ;;
	*)
	    echo unknown configuration: $name
	    exit 1
	    ;;
    esac


    if [ \! -d $VMDIR/$name ]; then
	git clone $repo $VMDIR/$name || exit 1
	pushd $VMDIR/$name
    else
	pushd $VMDIR/$name
	git pull
    fi	

    git checkout $commit  || (popd; exit 1)
}

function build() {
    name=$1
    debuglevel=$2

    nvmfile=$NVM_FILE
    
    xcflags="-march=native -Wno-all -Wno-deprecated-declarations"
    
    cflags_ccjava=
    cflags_ccjava="$cflags_ccjava -D OUR_PERSIST"
    cflags_ccjava="$cflags_ccjava -D NVM_FILE_PATH=$nvmfile"
    cflags_ccjava="$cflags_ccjava -D USE_NVM"
    cflags_ccjava="$cflags_ccjava -D ENABLE_NVM_WRITEBACK"
    cflags_ccjava="$cflags_ccjava -D USE_CLWB"
    
    case $name in
	none-original-none)
	    ;;
	none-*-*)
	    xcflags="$xcflags $cflags_ccjava"
	    xcflags="$xcflags -D OURPERSIST_DURABLEROOTS_ALL_FALSE"
	    ;;
	all-*-*)
	    xcflags="$xcflags $cflags_ccjava"
	    xcflags="$xcflags -D OURPERSIST_DURABLEROOTS_ALL_TRUE"
	    ;;
	*)
	    echo unknown configuration: $name
	    exit 1
	    ;;
    esac

    # other options
    case $name in
	*-cw1-*)
	    xcflags="$xcflags -D OUR_PERSIST_SINGLE_FENCE"
	    ;;
	*-cwx-*)
	    xcflags="$xcflags -D OUR_PERSIST_UNSAFE_NO_HANDSHAKE"
	    ;;
	*-wwx-*)
	    xcflags="$xcflags -D OUR_PERSIST_UNSAFE_NO_MFENCE"
	    ;;
	*)
	    ;;
    esac
    
    # analysis
    case $name in
	none-original-none)
	    ;;
	*-*-none)
	    xcflags="$xcflags -D OUR_PERSIST_ALWAYS_HANDSHAKE"
	    xcflags="$xcflags -D OUR_PERSIST_ALWAYS_BARRIER"
	    ;;
	*-*-b)
	    xcflags="$xcflags -D OUR_PERSIST_ALWAYS_HANDSHAKE"
	    ;;
	*-*-h)
	    xcflags="$xcflags -D OUR_PERSIST_ALWAYS_BARRIER"
	    ;;
	*-*-bh)
	    ;;
	*)
	    echo unknown configuration: $name
	    exit 1
	    ;;
    esac
	
    configure=
    configure="$configure -with-jvm-variants=server"
    configure="$configure -with-extra-cxxflags=-std=c++17"

    # checkout repo and pushd
    checkout_branch $name || exit 1

    ## Release build
    c="$configure -with-debug-level=release"
    bash configure $c -with-extra-cflags="$xcflags" || (popd; exit 1)
    make CONF=release || (popd; exit 1)

    ## Counter (fastdebug)
    case $name in
	*-original-* | *-cwx-*)
	    : # pass
	    ;;
	*)
	    c="$configure -with-debug-level=fastdebug"
	    xcflags="$xcflags -D NVM_COUNTER"
	    bash configure $c -with-extra-cflags="$xcflags" || (popd; exit 1)
	    make CONF=fastdebug || (popd; exit 1)
	    ;;
    esac
    
    popd
}

function main() {
    build none-original-none || exit 1
    build none-ww-none  || exit 1
    build none-wwx-none  || exit 1
    build none-cw-none  || exit 1

    build all-ww-none  || exit 1
    build all-cw-none  || exit 1
    build all-cw-h   || exit 1
    build all-cwx-none  || exit 1
    build all-cwx-h  || exit 1
}    

if [ x"$*" \!= x ]; then
    for i in $*; do
	build $i || exit 1
    done
else
    main
fi
