# execute this program in the "experiment" directory

import re
import sys
import os

LOG_DIR = "exec_logs"
OUT_DIR = "results"

DACAPO_WARMUP_PATTERN = r"===== DaCapo 9.12.*warmup \d+ in (\d+) msec ====="
DACAPO_FINAL_PATTERN = r"===== DaCapo 9.12.* PASSED in (\d+) msec ====="
IDS_PATTERN = r".*thread \d+ round.*time (\d+\.\d+)"
XML_PATTERN = r"time (\d+\.\d+)"

COUNTERS = [
    'alloc_dram_count',
    'alloc_dram_word' ,
    'alloc_nvm_count' ,
    'alloc_nvm_word' ,
    'persistent_count',
    'persistent_word',
    'ensure_recoverable',
    'handshake',
    'barrier_full',
    'barrier_async',
    'barrier_no'
]

COUNTER_PATTERN = {
#    '_alloc_dram_g':          'alloc_dram_count' ,
#    '_alloc_dram_word_g':     'alloc_dram_word' ,
    '_alloc_nvm_g':           'alloc_nvm_count' ,
    '_alloc_nvm_word_g':      'alloc_nvm_word' ,
    
    '_persistent_obj_g':      'persistent_count',
    '_persistent_obj_word_g': 'persistent_word',

    '_call_ensure_recoverable_g': 'ensure_recoverable',
    'handshake count':        'handshake',
    'full barrier count':     'barrier_full',
    'half barrier count':     'barrier_async',
    'no barrier count':       'barrier_no'
}

DACAPO = [
    "avrora",
    "fop",
    "h2",
    "jython",
    "luindex",
    "lusearch-fix",
    "lusearch",
    "pmd",
    "sunflow",
    "xalan"
]

IDS = [
    "HAMT-1",
    "HAMT-10",
    "HAMTS-1",
    "HAMTS-10",
    "CHAMP-1",
    "CHAMP-10"
]

XML = [
    "xml",
    "xml-custom-hashmap",
    "xml-tu"
]

NO_ANALYSIS = [
#    "fop",
#    "h2",
#    "jython",
#    "pmd"
]

def do_pick_dacapo(file_name):
    passed = False
    numbers = []
    with open(file_name) as f:
        for line in f.readlines():
            m = re.match(DACAPO_WARMUP_PATTERN, line)
            if m:
                numbers.append(int(m.group(1)))
                continue
            m = re.match(DACAPO_FINAL_PATTERN, line)
            if m:
                numbers.append(int(m.group(1)))
                passed = True
                break
    if not passed:
        raise Exception("DaCapo not passed: " + file_name)
    return numbers

def do_pick_ids(file_name):
    numbers = []
    with open(file_name) as f:
        for line in f.readlines():
            m = re.match(IDS_PATTERN, line)
            if m:
                numbers.append(float(m.group(1)) * 1000)
    return numbers

def do_pick_xml(file_name):
    numbers = []
    with open(file_name) as f:
        for line in f.readlines():
            m = re.match(XML_PATTERN, line)
            if m:
                numbers.append(float(m.group(1)) * 1000)
    return numbers


def pick(setting, vm, analysis, benchmark):
    p = {
        "s": setting,
        "v": vm,
        "a": analysis,
        "b": benchmark
    }
    log_file = "time_%(s)s-%(v)s-%(a)s_%(b)s.txt" % p
    out_file = "%(s)s_%(b)s_%(v)s_%(a)s.txt" % p

    if benchmark in DACAPO:
        nums = do_pick_dacapo(LOG_DIR + "/" + log_file)
    elif benchmark in IDS:
        nums = do_pick_ids(LOG_DIR + "/" + log_file)
    elif benchmark in XML:
        nums = do_pick_xml(LOG_DIR + "/" + log_file)
    else:
        raise Exception("bad benchmark: " + benchmark)

    with open(OUT_DIR + "/" + out_file, "w") as f:
        for n in nums:
            f.write(str(n))
            f.write("\n")

#print(do_pick_dacapo("../results/all-cw-bh_avrora.txt"))
#print(do_pick_ids("../results/all-cw-bh_CHAMP-1.txt"))

def manual():
    for s, v, a in [
            ("none", "original", "none"),
            ("none", "ww", "none"),
            ("none", "cw1", "none"),
            ("all", "ww", "none"),
            ("all", "cw1", "none"),
            ("all", "cw1", "h"),
            ("all", "cw1", "bh"),
            ("all", "cw", "none"),
            ("all", "cw", "h"),
            ("all", "cw", "bh")]:
        for b in DACAPO + IDS:
            if a != "none" and b in NO_ANALYSIS:
                continue
            pick(s, v, a, b)


def pick_time():
    files=os.listdir(LOG_DIR)
    for i in files:
        m = re.match(r'time_(.*)-(.*)-(.*)_(.*)\.txt', i)
        if m:
            s = m.group(1)
            v = m.group(2)
            a = m.group(3)
            b = m.group(4)
            pick(s, v, a, b)


def count_pick1(setting, vm, analysis, benchmark):
    def pick_single_file(file_name):
        numbers = {}
        with open(LOG_DIR + "/" + file_name) as f:
            for line in f.readlines():
                m = re.match(r"\[NVMCounter\] (.*): *(.*)", line)
                if m:
                    for k in COUNTER_PATTERN.keys():
                        if m.group(1) == k:
                            numbers[COUNTER_PATTERN[k]] = m.group(2)
        return numbers
    
    p = {
        "s": setting,
        "v": vm,
        "a": analysis,
        "b": benchmark
    }
    log_file1 = "count1_%(s)s-%(v)s-%(a)s_%(b)s.txt" % p
    log_file2 = "count2_%(s)s-%(v)s-%(a)s_%(b)s.txt" % p
    out_file = "count_%(s)s-%(v)s-%(a)s_%(b)s.txt" % p

    iter1 = pick_single_file(log_file1)
    iter2 = pick_single_file(log_file2)

    with open(OUT_DIR + "/" + out_file, "w") as f:
        for k in COUNTERS:
            if k in iter1:
                f.write(iter1[k])
            else:
                f.write("-")
            f.write("\t")
            if k in iter2:
                f.write(iter2[k])
            else:
                f.write("-")
            f.write("\t")
            f.write(k)
            f.write('\n')
            
def pick_count():
    files=os.listdir(LOG_DIR)
    for i in files:
        m = re.match(r'count1_(.*)-(.*)-(.*)_(.*)\.txt', i)
        if m:
            s = m.group(1)
            v = m.group(2)
            a = m.group(3)
            b = m.group(4)
            count_pick1(s, v, a, b)
    
            
def main():
    os.makedirs(OUT_DIR, exist_ok=True)
    pick_time()
    pick_count()
            
main()
