# execute this program in "experiment" directory

ANALYSIS_DIR=${ANALYSIS_DIR:=analysis/results-v3}
DACAPO_JAR=${DACAPO_DIR:=dacapo-9.12-MR1-bach.jar}
OUT_DIR=${LOG_DIR:=exec_logs}

CCJAVA_BENCHMARK_DIR=ccjava-benchmarks
#TASKSET="taskset --cpu-list 18-35,54-71"
TASKSET=""

echo $ANALYSIS_DIR

DACAPO_ITER=30
IDS_ITER=30
XML_ITER=30

function run_any() {
    name=$1
    program=$2
    analysis=$3
    cmdline=$4
    mode=$5

    vmopt=
    vmopt="$vmopt -XX:-UseCompressedOops"
    vmopt="$vmopt -XX:-UseCompressedClassPointers"
    vmopt="$vmopt -XX:+UseSerialGC"
    vmopt="$vmopt -XX:+TieredCompilation"
    vmopt="$vmopt -XX:TieredStopAtLevel=1"

    case $name in
	*-*-b | *-*-h | *-*-bh)
	    analysis_path=${ANALYSIS_DIR}/${analysis}_out/
	    if [ \! -d $analysis_path ]; then
		echo skipping $name $program
		return
	    fi
	    vmopt="$vmopt -XX:AnalysisPath=${analysis_path}"
	    ;;
	*-*-none)
	    ;;
	*)
	    echo unknown configuration: $name
	    exit 1
	    ;;
    esac

    case $name in
	*-*-b)
	    vmopt="$vmopt -XX:+CCJavaEliminateBarrier"
	    ;;
	*-*-h)
	    vmopt="$vmopt -XX:+CCJavaEliminateHandshake"
	    ;;
	*-*-bh)
	    vmopt="$vmopt -XX:+CCJavaEliminateBarrier"
	    vmopt="$vmopt -XX:+CCJavaEliminateHandshake"
	    ;;
	*-*-none)
	    ;;
	*)
	    echo unknown configuration: $name
	    exit 1
	    ;;
    esac

    case $name in
	*-cw1-* | *-cw-*)
#	    vmopt="$vmopt -XX:+CCJavaEliminateVerbose"
	    ;;
	*)
	    ;;
    esac

    case $mode in
	time)
	    debuglevel=release
	    ;;
	count1 | count2)
	    debuglevel=fastdebug
	    ;;
	*)
	    echo unknown mode $mode
	    exit 1
	    ;;
    esac

    jvm=vms/$name/build/linux-x86_64-server-$debuglevel/jdk/bin/java

    output_file=$OUT_DIR/${mode}_${name}_$program.txt
    
    date
    echo $TASKSET $jvm $vmopt $cmdline
    echo $TASKSET $jvm $vmopt $cmdline > $output_file
    # $TASKSET $jvm $vmopt $cmdline >> $output_file 2>&1
    $TASKSET $jvm $vmopt $cmdline 2>&1 | tee -a $output_file
}


function run_dacapo() {
    name=$1
    program=$2
    mode=$3

    case $mode in
	time)
	    iter=$DACAPO_ITER
	    ;;
	count1)
	    iter=1
	    ;;
	count2)
	    iter=2
	    ;;
    esac
    
    run_any $name $program $program "-jar $DACAPO_JAR -n $iter $program" $mode
}

function time_dacapo() {
    run_dacapo $1 $2 time
}

function count_dacapo() {
    run_dacapo $1 $2 count1
    run_dacapo $1 $2 count2
}

function run_ids() {
    name=$1
    program=$2
    mode=$3

    case $program in
	HAMT-*)
	    dirname=hamt
	    ;;
	HAMTS-*)
	    dirname=hamts
	    ;;
	CHAMP-*)
	    dirname=capsule-map
	    ;;
	*)
	    echo unknown benchmark $program
	    exit 1
	    ;;
    esac
    batch=${program#*-}
    
    case $mode in
	time)
	    iter=$IDS_ITER
	    ;;
	count1)
	    iter=1
	    ;;
	count2)
	    iter=2
	    ;;
    esac
   
    cmdline="-cp $CCJAVA_BENCHMARK_DIR/$dirname/src Benchmark -b $batch -r $iter"
    run_any $name $program $dirname "$cmdline" $mode
}

function time_ids() {
    run_ids $1 $2 time
}

function count_ids() {
    run_ids $1 $2 count1
    run_ids $1 $2 count2
}

function run_xml() {
    name=$1
    program=$2
    mode=$3
    dirname=$2
    xmldata=$CCJAVA_BENCHMARK_DIR/$dirname/data/psd7003.xml

    case $mode in
	time)
	    iter=$XML_ITER
	    ;;
	count1)
	    iter=1
	    ;;
	count2)
	    iter=2
	    ;;
    esac

    opt="-r $iter -t 30"
    cmdline="-cp $CCJAVA_BENCHMARK_DIR/$dirname/bin Main $opt $xmldata"
    run_any $name $program $dirname "$cmdline" $mode
}

function time_xml() {
    run_xml $1 $2 time
}

function count_xml() {
    run_xml $1 $2 count1
    run_xml $1 $2 count2
}

#DACAPO="avrora fop h2 jython luindex lusearch lusearch-fix pmd sunflow xalan"
DACAPO="avrora fop h2 jython luindex lusearch pmd sunflow xalan"

function run_time() {
    VMS=
    VMS="$VMS none-original-none"
    VMS="$VMS none-ww-none none-cw-none"
    VMS="$VMS all-ww-none all-cw-none all-cw-h" # all-cw-b all-cw-bh"
    for i in $VMS
    do
	for j in $DACAPO
	do
 	    time_dacapo $i $j
	done
	time_ids $i HAMT-1 hamt 1
	time_ids $i HAMT-10 hamt 10
	time_ids $i CHAMP-1 capsule-map 1
	time_ids $i CHAMP-10 capsule-map 10
	time_xml $i xml-tu
    done
}

function run_count() {
    VMS=
    VMS="$VMS none-ww-none none-cw-none"
    VMS="$VMS all-ww-none all-cw-none all-cw-h" # all-cw-b all-cw-bh"
    for i in $VMS
    do
	for j in $DACAPO
	do
 	    count_dacapo $i $j
	done
	count_ids $i HAMT-1 hamt 1
	count_ids $i HAMT-10 hamt 10
	count_ids $i CHAMP-1 capsule-map 1
	count_ids $i CHAMP-10 capsule-map 10
	count_xml $i xml-tu
    done
}

function run_breakdown() {
    VMS=
    VMS="$VMS none-wwx-none"
    VMS="$VMS all-cwx-none all-cwx-h" #all-cwx-b all-cwx-bh"
    for i in $VMS
    do
	for j in $DACAPO
	do
 	    time_dacapo $i $j
	done
	time_ids $i HAMT-1 hamt 1
	time_ids $i HAMT-10 hamt 10
	time_ids $i CHAMP-1 capsule-map 1
	time_ids $i CHAMP-10 capsule-map 10
	time_xml $i xml-tu
    done
}

function main() {
    run_time
    run_count
    run_breakdown
}

function run_single() {
    mode=$1
    vm=$2
    prog=$3

    case "$3" in
	HAMT-1)
	    ${mode}_ids $vm HAMT-1 hamt 1
	    ;;
	HAMT-10)
	    ${mode}_ids $vm HAMT-10 hamt 10
	    ;;
	CHAMP-1)
	    ${mode}_ids $vm CHAMP-1 capsule-map 1
	    ;;
	CHAMP-10)
	    ${mode}_ids $vm CHAMP-10 capsule-map 10
	    ;;
	XML)
	    ${mode}_xml $vm xml-tu
	    ;;
	*)
	    for j in $DACAPO
	    do
		if [ $prog = $j ]
		then
 		    ${mode}_dacapo $vm $j
		    return
		fi
	    done
	    echo "unknown benchmark program: $prog"
    esac
}
    
while [ x"$*" \!= x ]; do
    case "$1" in
	-small)
	    DACAPO_ITER=3
	    IDS_ITER=3
	    XML_ITER=3
	    shift
	    ;;
	-normal)
	    DACAPO_ITER=15
	    IDS_ITER=15
	    XML_ITER=15
	    shift
	    ;;
	-large)
	    DACAPO_ITER=30
	    IDS_ITER=30
	    XML_ITER=30
	    shift
	    ;;
	*)
	    break
	    ;;
    esac
done

mkdir -p $OUT_DIR
if [ x"$*" \!= x ]; then
    case "$1" in
	time)
	    mode=time
	    shift
	    ;;
	count)
	    mode=count
	    shift
	    ;;
	*)
	    echo "run_benchmark.sh (time|count) benchmark vm vm ..."
	    exit 1
	    ;;
    esac
    
    progs=$1
    shift
    if [ $progs = "all" ]; then
	progs="$DACAPO HAMT-1 HAMT-10 CHAMP-1 CHAMP-10 XML"
    fi
    
    for vm in $*; do
	for prog in $progs
	do
	    run_single $mode $vm $prog
	done
    done
else
    main
fi
